/**
 *
 * Magnolia Criteria API (http://www.openmindlab.com/lab/products/mgnlcriteria.html)
 * Copyright (C)2009 - 2010, Openmind S.r.l. http://www.openmindonline.it
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.sourceforge.openutils.mgnlcriteria.jcr.query;

import info.magnolia.cms.core.HierarchyManager;
import info.magnolia.cms.util.DeprecationUtil;
import net.sourceforge.openutils.mgnlcriteria.advanced.impl.AdvancedCriteriaImpl;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.xpath.impl.MagnoliaCriteriaImpl;
import net.sourceforge.openutils.mgnlcriteria.jcr.query.xpath.impl.MagnoliaCriteriaWithLimitImpl;


/**
 * Factory for criteria queries. Since mgnlcriteria 2.x always use the simple createCriteria() factory method.
 * @author fgrilli
 * @author fgiust
 * @author diego_schivo
 * @version $Id: JCRCriteriaFactory.java 2923 2010-08-29 08:58:29Z fgiust $
 */
public final class JCRCriteriaFactory
{

    private JCRCriteriaFactory()
    {
    }

    /**
     * Creates a query criteria for dynamic query composition. All the details can be set on the Criteria instance
     * returned.
     * @return an AdvancedCriteriaImpl
     */
    public static Criteria createCriteria()
    {
        return new AdvancedCriteriaImpl();
    }

    /**
     * Creates a jcr query using the specified statement.
     * @param hm the HierarchyManager
     * @param query the statement of the query
     * @param language the language of the query
     * @return a DirectJcrQuery
     */
    public static DirectJcrQuery createDirectJcrQuery(HierarchyManager hm, String query, String language)
    {
        return new DirectJcrQuery(hm, query, language);
    }

    /**
     * A factory for Magnolia CMS specific Criteria implementation <br>
     * <strong>Warning: the object returned by this method is not thread-safe</strong>To use it as a shared global
     * variable, client-code must coordinate access to it.
     * @param path String - the path in JCR repository to search for Nodes. This is the path which precedes the
     * predicate
     * @param queryManager - an instance of info.magnolia.cms.core.search.QueryManager
     * @param itemType String - if not specified defaults to "mgnl:content"
     * @return a factory for Magnolia CMS specific Criteria implementation
     * @deprecated please use createCriteria()
     */
    @Deprecated
    public static Criteria createMgnlCriteria(String path, info.magnolia.cms.core.search.QueryManager queryManager,
        String itemType)
    {
        DeprecationUtil.isDeprecated("please use createCriteria() instead");
        return new MagnoliaCriteriaImpl(path, queryManager, itemType);
    }

    /**
     * A factory for Magnolia CMS specific Criteria implementation <br>
     * <strong>Warning: the object returned by this method is not thread-safe</strong>To use it as a shared global
     * variable, client-code must coordinate access to it.
     * @param path String - the path in JCR repository to search for Nodes. This is the path which precedes the
     * predicate
     * @param queryManager - an instance of info.magnolia.cms.core.search.QueryManager
     * @param itemType String - if not specified defaults to "mgnl:content"
     * @param clazz Class<?> - the list method of this implementation must return instances of this type
     * @return a factory for Magnolia CMS specific Criteria implementation
     * @deprecated please use createCriteria()
     */
    @Deprecated
    public static Criteria createMgnlCriteria(String path, info.magnolia.cms.core.search.QueryManager queryManager,
        String itemType, Class< ? > clazz)
    {
        DeprecationUtil.isDeprecated("please use createCriteria() instead");
        return new MagnoliaCriteriaImpl(path, queryManager, itemType, clazz);
    }

    /**
     * A factory for Magnolia CMS specific Criteria implementation which can return a subset of results instead of the
     * whole resultset, much like a MySQL limit clause. For this to work, the underlying JCR repository implementation
     * must support this feature (Jackrabbit 1.4+ does).<br>
     * To set the maximum number of returned items you must call {@link Criteria#setMaxResults(int)} (by default
     * {@link Criteria#setFirstResult(int)} is set to 0).<br>
     * Please notice that if {@link Criteria#setMaxResults(int)} is not explicitly called before the
     * {@link Criteria#list()} method, the latter will return the entire resultset as in
     * {@link JCRCriteriaFactory#createMgnlCriteria(String, info.magnolia.cms.core.search.QueryManager, String)} <br>
     * <strong>Warning: the object returned by this method is not thread-safe</strong>To use it as a shared global
     * variable, client-code must coordinate access to it.
     * @param path String - the path in JCR repository to search for Nodes. This is the path which precedes the
     * predicate
     * @param queryManager - an instance of info.magnolia.cms.core.search.QueryManager
     * @param itemType String - if not specified defaults to "mgnl:content"
     * @return a factory for Magnolia CMS specific Criteria implementation
     * @deprecated please use createCriteria()
     */
    @Deprecated
    public static Criteria createMgnlCriteriaWithLimit(String path,
        info.magnolia.cms.core.search.QueryManager queryManager, String itemType)
    {
        DeprecationUtil.isDeprecated("please use createCriteria() instead");
        return new MagnoliaCriteriaWithLimitImpl(path, queryManager, itemType);
    }

    /**
     * A factory for Magnolia CMS specific Criteria implementation which can return a subset of results instead of the
     * whole resultset, much like a MySQL limit clause. For this to work, the underlying JCR repository implementation
     * must support this feature (Jackrabbit 1.4+ does).<br>
     * To set the maximum number of returned items you must call {@link Criteria#setMaxResults(int)} (by default
     * {@link Criteria#setFirstResult(int)} is set to 0).<br>
     * Please notice that if {@link Criteria#setMaxResults(int)} is not explicitly called before the
     * {@link Criteria#list()} method, the latter will return the entire resultset as in
     * {@link JCRCriteriaFactory#createMgnlCriteria(String, info.magnolia.cms.core.search.QueryManager, String, Class)} <br>
     * <strong>Warning: the object returned by this method is not thread-safe</strong>To use it as a shared global
     * variable, client-code must coordinate access to it.
     * @param path String - the path in JCR repository to search for Nodes. This is the path which precedes the
     * predicate
     * @param queryManager - an instance of info.magnolia.cms.core.search.QueryManager
     * @param itemType String - if not specified defaults to "mgnl:content"
     * @param clazz Class<?> - the list method of this implementation must return instances of this type
     * @return a factory for Magnolia CMS specific Criteria implementation
     * @deprecated please use createCriteria()
     */
    @Deprecated
    public static Criteria createMgnlCriteriaWithLimit(String path,
        info.magnolia.cms.core.search.QueryManager queryManager, String itemType, Class< ? > clazz)
    {

        DeprecationUtil.isDeprecated("please use createCriteria() instead");
        return new MagnoliaCriteriaWithLimitImpl(path, queryManager, itemType, clazz);
    }

}
